"""Tools to work with files
"""
import os
import shutil
import glob
import zipfile
import logging
from os.path import join as opj

def create_dir(dir_path):
    """Returns the directory **dir_path** and create it if path does not exist.

    Args:
        dir_path (str): Path to the directory that will be created.

    Returns:
        str: Directory dir path.
    """
    if not os.path.exists(dir_path):
        try:
            os.makedirs(dir_path)
        except:
            pass
    return dir_path

def get_workflow_path(workflow_path):
    """Return the directory **workflow_path** and create it if workflow_path
    does not exist. If **workflow_path** exists a consecutive numerical suffix
    is added to the end of the **workflow_path** and is returned.

    Args:
        workflow_path (str): Path to the workflow results.

    Returns:
        str: Path to the workflow results directory.
    """
    if not os.path.exists(workflow_path):
        return workflow_path

    cont = 1
    while os.path.exists(workflow_path):
        workflow_path = workflow_path.rstrip('\\/0123456789_') + '_' + str(cont)
        cont += 1
    return workflow_path

def zip_list(zip_file, file_list, out_log=None):
    """ Compress all files listed in **file_list** into **zip_file** zip file.

    Args:
        zip_file (str): Output compressed zip file.
        file_list (:obj:`list` of :obj:`str`): Input list of files to be compressed.
    """
    with zipfile.ZipFile(zip_file, 'w') as zip:
        for f in file_list:
            zip.write(f, arcname=os.path.basename(f))
    if out_log:
        out_log.info("Adding:")
        out_log.info(str(file_list))
        out_log.info("to: "+ os.path.abspath(zip_file))

def unzip_list(zip_file, out_log=None):
    """ Extract all files in the zipball file and return a list containing the
        absolute path of the extracted files.

    Args:
        zip_file (str): Input compressed zip file.

    Returns:
        :obj:`list` of :obj:`str`: List of paths of the extracted files.
    """
    with zipfile.ZipFile(zip_file, 'r') as zip:
        zip.extractall()
        file_list = [os.path.abspath(f) for f in zip.namelist()]

    if out_log:
        out_log.info("Extracting: "+ os.path.abspath(zip_file))
        out_log.info("to:")
        out_log.info(str(file_list))

    return file_list

def zip_top(zip_file, out_log=None):
    """ Compress all *.itp and *.top files in the cwd into **zip_file** zip file.

    Args:
        zip_file (str): Output compressed zip file.
    """
    ext_list = [".itp", ".top"]
    file_list = [f for f in os.listdir(os.getcwd()) if os.path.isfile(f) and os.path.splitext(f)[1] in ext_list]
    zip_list(zip_file, file_list, out_log)

def unzip_top(zip_file, top_file, out_log=None):
    """ Extract all files in the zip_file and copy the file extracted ".top" file to top_file.

    Args:
        zip_file (str): Input topology zipball file path.
        top_file (str): Output ".top" file where the extracted ".top" file will be copied.

    Returns:
        :obj:`list` of :obj:`str`: List of extracted files paths.
    """
    top_list = unzip_list(zip_file, out_log)
    original_top = next(name for name in top_list if name.endswith(".top"))
    shutil.move(original_top, top_file)
    if out_log:
        out_log.info("Moving: "+ os.path.abspath(original_top) +' to: '+ os.path.abspath(top_file))

    return top_list


def get_logs_prefix():
    return 22*' '

def get_logs(path=None, prefix=None, step=None, console=False, level='INFO'):
    """ Get the error and and out Python Logger objects.

    Args:
        path (str): (current working directory) Path to the log file directory.
        prefix (str): Prefix added to the name of the log file.
        step (str):  String added between the **prefix** arg and the name of the log file.
        console (bool): (False) If True, show log in the execution terminal.
        level (str): ('INFO') Set Logging level. ['CRITICAL','ERROR','WARNING','INFO','DEBUG','NOTSET']

    Returns:
        :obj:`tuple` of :obj:`Logger` and :obj:`Logger`: Out and err Logger objects.
    """
    path = path if path else os.getcwd()
    out_log_path = create_name(path=path, prefix=prefix, step=step, name='log.out')
    err_log_path = create_name(path=path, prefix=prefix, step=step, name='log.err')

    # Create dir if it not exists
    create_dir(os.path.dirname(os.path.abspath(out_log_path)))

    # Create logging format
    logFormatter = logging.Formatter("%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s]  %(message)s")

    # Create logging objects
    out_Logger = logging.getLogger(out_log_path)
    err_Logger = logging.getLogger(err_log_path)


    #Create FileHandler
    out_fileHandler = logging.FileHandler(out_log_path, mode='a', encoding=None, delay=False)
    err_fileHandler = logging.FileHandler(err_log_path, mode='a', encoding=None, delay=False)

    # Asign format to FileHandler
    out_fileHandler.setFormatter(logFormatter)
    err_fileHandler.setFormatter(logFormatter)

    #Asign FileHandler to logging object
    out_Logger.addHandler(out_fileHandler)
    err_Logger.addHandler(err_fileHandler)

    # Create consoleHandler
    consoleHandler = logging.StreamHandler()
    # Asign format to consoleHandler
    consoleHandler.setFormatter(logFormatter)

    # Asign consoleHandler to logging objects as aditional output
    if console:
        out_Logger.addHandler(consoleHandler)
        err_Logger.addHandler(consoleHandler)

    # Set logging level level
    out_Logger.setLevel(level)
    err_Logger.setLevel(level)
    return out_Logger, err_Logger

def human_readable_time(time_ps):
    """Transform **time_ps** to a human readable string.

    Args:
        time_ps (int): Time in pico seconds.

    Returns:
        str: Human readable time.
    """
    time_units = ['femto seconds','pico seconds','nano seconds','micro seconds','mili seconds']
    time = time_ps * 1000
    for tu in time_units:
        if time < 1000:
            return str(time)+' '+tu
        else:
            time = time/1000
    return str(time_ps)

def create_name(path=None, prefix=None, step=None, name=None):
    """ Return file name.

    Args:
        path (str): Path to the file directory.
        prefix (str): Prefix added to the name of the file.
        step (str):  String added between the **prefix** arg and the **name** arg of the file.
        name (str): Name of the file.

    Returns:
        str: Composed file name.
    """
    name = '' if name is None else name.strip()
    if step:
        if name:
            name = step+'_'+name
        else:
            name = step
    if prefix:
        if name:
            name = prefix+'_'+name
        else:
            name = prefix
    if path:
        if name:
            name = opj(path, name)
        else:
            name = path
    return name

def write_failed_output(file_name):
    with open(file_name, 'w') as f:
        f.write('Error\n')



"""Boiler plate functions for testsys
"""
import os
import sys
import shutil
from os.path import join as opj
from biobb_common.configuration import settings
from biobb_common.tools import file_utils as fu

def test_setup(test_object, dict_key):
    """Add the unitest_dir, test_dir, conf_file_path, system, properties and path as
    attributes to the **test_object** and create a directory to launch the unitest.

    Args:
        test_object (:obj:`test`): The test object.
        dict_key (str): Key of the test parameters in the yaml config file.
    """
    test_object.testfile_dir = os.path.dirname(os.path.abspath(sys.modules[test_object.__class__.__module__].__file__))
    test_object.unitest_dir = os.path.dirname(test_object.testfile_dir)
    test_object.test_dir = os.path.dirname(test_object.unitest_dir)
    test_object.data_dir = opj(test_object.test_dir,'data')
    test_object.conf_file_path= opj(test_object.test_dir, 'conf.yml')
    test_object.system=os.getenv('testsys')
    if test_object.system is None:
        print 'WARNING: "testsys" env variable should be set, "linux" will be used by default value.'
        print '     Please, try: "export testsys=linux"'
        test_object.system='linux'
    conf = settings.YamlReader(test_object.conf_file_path, test_object.system)
    test_object.properties = conf.get_prop_dic()[dict_key]
    test_object.paths = {k:v.replace('test_data_dir', test_object.data_dir, 1) for k, v in conf.get_paths_dic()[dict_key].iteritems()}
    fu.create_dir(test_object.properties['path'])
    os.chdir(test_object.properties['path'])

def test_teardown(test_object):
    """Remove the **test_object.properties['workflow_path']**

    Args:
        test_object (:obj:`test`): The test object.
    """
    shutil.rmtree(test_object.properties['workflow_path'])

def exe_success(return_code):
    """Check if **return_code** is 0

    Args:
        return_code (int): Return code of a process.

    Returns:
        bool: True if return code is equal to 0
    """
    return return_code == 0

def not_empty(file_path):
    """Check if file exists and is not empty.

    Args:
        file_path (str): Path to the file.

    Returns:
        bool: True if **file_path** exists and is not empty.
    """
    return ( os.path.isfile(file_path) and os.path.getsize(file_path) > 0 )



#!/usr/bin/env python

"""Settings loader module.

This module contains the classes to read the different formats of the
configuration files.
"""

import yaml
import os
from os.path import join as opj
from biobb_common.tools import file_utils as fu

class YamlReader(object):
    """Configuration file loader for yaml format files.

    Args:
        conf_file_path (str): Path to the configuration YAML file.
        system (str): System name from the systems section in the configuration file.
    """

    def __init__(self, conf_file_path, system):
        self.conf_file_path= os.path.abspath(conf_file_path)
        self.system = system
        self.properties = self._read_yaml()
        self.properties[system]['workflow_path'] = fu.get_workflow_path(self.properties[system]['workflow_path'])

    def _read_yaml(self):
        with open(self.conf_file_path, 'r') as stream:
            return yaml.safe_load(stream)

    def get_prop_dic(self, prefix=None, global_log=None):
        """get_prop_dic() returns the properties dictionary where keys are the
        step names in the configuration YAML file and every value contains another
        nested dictionary containing the keys and values of each step properties section.
        All the paths in the system section are copied in each nested dictionary.
        For each nested dictionary the following keys are added:
            | **path** (*str*): Absolute path to the step working dir.
            | **step** (*str*): Name of the step.
            | **prefix** (*str*): Prefix if provided.
            | **global_log** (*Logger object*): Log from the main workflow.

        Args:
            prefix (str): Prefix if provided.
            global_log (:obj:Logger): Log from the main workflow.

        Returns:
            dict: Dictionary of properties.
        """
        prop_dic = dict()
        prefix = '' if prefix is None else prefix.strip()

        for key in self.properties:
            if isinstance(self.properties[key], dict):
                if 'paths' in self.properties[key] or 'properties' in self.properties[key]:
                    prop_dic[key] = dict()
                    prop_dic[key]['path']=fu.create_name(path=self.properties[self.system]['workflow_path'], prefix=prefix, step=key)
                    prop_dic[key]['step']= key
                    prop_dic[key]['prefix']= prefix
                    prop_dic[key]['global_log']= global_log
                    prop_dic[key]['system']= self.system
                    prop_dic[key].update(self.properties[self.system].copy())
                if 'properties' in self.properties[key] and isinstance(self.properties[key]['properties'], dict):
                    prop_dic[key].update(self.properties[key]['properties'].copy())
                    if self.properties[self.system].get('log_level', None):
                        prop_dic[key]['log_level']= self.properties[self.system]['log_level']


        return prop_dic

    def get_paths_dic(self, prefix=None):
        """get_paths_dic() returns the paths dictionary where keys are the
        step names in the configuration YAML file and every value contains another
        nested dictionary containing the keys and values of each step paths section.
        All the paths starting with 'dependency' are resolved. If the path starts
        with the string 'file:' nothing is done, however if the path starts with
        any other string path is prefixed with the absolute step path.

        Args:
            prefix (str): Prefix if provided.

        Returns:
            dict: Dictionary of paths.
        """
        prop_dic = dict()
        prefix = '' if prefix is None else prefix.strip()
        #Filtering just paths
        for key in self.properties:
            if isinstance(self.properties[key], dict):
                if 'paths' in self.properties[key]:
                    prop_dic[key]=self.properties[key]['paths'].copy()

        #Solving dependencies and adding workflow and step path
        for key in prop_dic:
            for key2, value in prop_dic[key].iteritems():
                if isinstance(value, basestring) and value.startswith('dependency'):
                    while isinstance(value, basestring) and value.startswith('dependency'):
                        dependency_step=value.split('/')[1]
                        value = prop_dic[value.split('/')[1]][value.split('/')[2]]
                    prop_dic[key][key2] = opj(self.properties[self.system]['workflow_path'], prefix, dependency_step, value)
                elif isinstance(value, basestring) and value.startswith('file:'):
                    prop_dic[key][key2] = value.split(':')[1]
                else:
                    prop_dic[key][key2] = opj(self.properties[self.system]['workflow_path'], prefix, key, value)

        return prop_dic
